/*	harp.c */
#ifdef HARP
#ifndef IPSEC  /* if we don't have IPSEC, do MIP style security */
#define MIPSEC
#endif

#include <sys/param.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <fcntl.h>
#include <sys/time.h>
#include <sys/types.h>

#include "harp.h"
#include "mip_log.h"
#include "mipconfig.h"
#include "route.h"
#include "vlist.h"
#include "udp_reg.h"
#include "faha_pinfo.h"
#ifdef IPSEC
#include "ipsec.h"
#include <netsec/ipsec.h>

int ipsec_level = IPSEC_LEVEL_REQUIRE;
int ipsec_type = SO_SECURITY_AUTHENTICATION;
#endif


struct sockaddr_in harpRegRem;  /* remote IP ( co-HA ) */
struct sockaddr_in harpRegLoc;  /* local IP ( this HA ) */

extern struct mip_config mip_config;
extern struct vlist *hahead;
extern int debug;
extern harpSocket;
int harp_ackTimer = HARP_ACK_TIMELIMIT;
static int harp_sendSock;
int harp2 = FALSE;

/* authentication variables, not exported, except for the key itself */
#ifdef MIPSEC
char *haha_key;	/* haha Md5 authentication key */
static u_long haha_spi;
static int haha_keysize;
static int haha_transform;
static u_long dummy;
#endif

#ifdef MIPSEC
/* external authentication functions */
extern char* storeMd5( char*, int, unsigned char*, char* );  
extern char* computeMd5( char*, int, char*, char* );
extern char* get_key( struct keylist*, u_long, int*, u_long*, int*, u_long* );
#endif

int initHarpSocket( int );	/* open TCP connection to co-HA */
u_long GetHaIp();		/* get HA ip address */
void recvHarpDump( int );	/* request registration dump from co-HA */
void harpRegDump( int );	/* dump registration information to co-HA */
void vlist_to_reg( struct vlist, struct udp_register* );
void harpReadSock( int, char*, int );	/* read from harp TCP socket */
void harpForward( int, struct packet_info *, int );
void harp2ArpOff();
void harp2ArpOn();
void harp2Arp(int);
extern void haUdp(struct mip_ifs *, struct sockaddr_in *, struct packet_info *, int);
extern void sendOneUdp(int , char *, int , struct in_addr, u_short);
extern void set_arp( char*, char*, short );
extern int dumpme( char*, int );
#ifdef IPSEC
void vlist_to_ipsec( struct vlist, struct mse_ipsecext* );
#endif

/* open TCP connection with co-HA. & create UDP send port */
int initHarpSocket( int port_num )
{
	int sd;
	int rc;
	struct timeval time_out;  /* socket timeout limit */

#ifdef MIPSEC
	/* get haha authentication key */
	haha_key = get_key( mip_config.sa[HAHASECURITY].kl, 
				mip_config.coha.sin_addr.s_addr, 
				&haha_keysize, &haha_spi, &haha_transform,
				&dummy );
#endif

	/* create udp send socket */
	harp_sendSock = socket( AF_INET, SOCK_DGRAM, IPPROTO_UDP );
	if ( harp_sendSock < 0 ){
		mip_log( MLOG_ERR, "initHarpSocket(): failure in opening HARP
			send socket\n" );
		return ERROR;
	}
#ifdef IPSEC
	rc = setsockopt( harp_sendSock, SOL_SOCKET, 
		ipsec_type,
		(char*)&ipsec_level, sizeof(int));
	if ( rc < 0 ){
		perror( "HARP:setsockopt(IPSEC) failed in initHarpSocket()");
		exit(1);
	}
#endif

	/* set timeout limits */
	time_out.tv_sec = 5;  
	time_out.tv_usec = 0;

	/* configure TCP connection at remote end  */
	harpRegRem.sin_family = AF_INET;
	harpRegRem.sin_port = htons( port_num );
	harpRegRem.sin_addr.s_addr = mip_config.coha.sin_addr.s_addr;

	/* configure TCP connection at local end */
	harpRegLoc.sin_family = AF_INET;
	harpRegLoc.sin_port = htons( port_num );
	harpRegLoc.sin_addr.s_addr = htonl( INADDR_ANY );

	/*  create HARP registration socket */
	if (( sd = socket( AF_INET, SOCK_STREAM, IPPROTO_TCP ))< 0 ){
		mip_log( MLOG_ERR, "initHarpSocket(): harp TCP socket failed\n" );
		return ERROR;
	}
#ifdef IPSEC
	rc = setsockopt( sd, SOL_SOCKET, ipsec_type,
		(char*)&ipsec_level, sizeof(int));
	if ( rc < 0 ){
		perror( "setsockopt(IPSEC ENCRYPT) failed in initHarpSocket()");
		exit(1);
	}
#endif
		
	/* set socket to non-blocking */
	rc = setsockopt( sd, SOL_SOCKET, SO_RCVTIMEO, &time_out, sizeof( struct timeval ));
	if ( rc < 0 ){
		perror( "setsockopt failed in initHarpSocket()" );
		exit(1);
	} 

	/*  we are booting, so we will attempt to get a *
	*	registration dump from our co-HA 	*/
	if ( connect( sd, (struct sockaddr*)&harpRegRem, 
		sizeof(struct sockaddr_in)) > ERROR ){
			if ( debug ){
				printf( "HARP:initHarpSocket(): connecting to co-HA\n" );
			}
			recvHarpDump( sd );
	} 
	else if ( debug ){
		printf( "HARP:initHarpSocket(): failed to get reg dump from co-HA\n" );
	}
	close( sd );
	if (( sd = socket( AF_INET, SOCK_STREAM, IPPROTO_TCP ))< 0 ){  
		mip_log( MLOG_ERR, "HARP:initHarpSocket(): harp TCP socket failed\n" );
		return ERROR;
	}

	/* bind to socket and wait to do Reg Dump for our co-HA */
	if ( bind( sd, (struct sockaddr*)&harpRegLoc, sizeof(harpRegLoc))< 0 ){
		mip_log( MLOG_ERR, "initHarpSocket(): harp bind failed\n" );
		perror("bind failed");
		exit(1);
	}
	listen( sd, 1 );
	if ( debug ){
		printf("HARP:initHarpSocket(): bound and listening for co-HA requests\n" );
	}

	return sd;
}

/* 	Get ip address of HA interface, Used by Harp Ping & Ack */
u_long getHaIp()
{
	int i;
	for ( i=0; i< MAX_INTERFACES; i++){
		if ( mip_config.mip_ifs[i].amfa == 0 ){/* found HA interface */
			return (mip_config.mip_ifs[i].if_sin.sin_addr.s_addr );
		}
	}
	return 0;
}

/*	Dump our registration table down HARP tcp connection 
    0                   1                   2                   3
    0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
   |     type=HARP_REG_DUMP          |    size=8                   |
   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
   |                   Number of Registrations                     |
   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
   |                   Size of Registrations                       |
   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
   |                  Mobile-IP registration field ...            |
   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
   |                  Mobile-IP registration field ...            |
   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
   |              etc.                             | */

void harpRegDump( int s )
{
	int newsd;	/* socket descriptor */
	int sz;		/* size of sockaddr */
	unsigned long count=0, reg_size; /* number of, size of registrations */ 
	unsigned short type, size;
	struct vlist *vptr; 
	char packet[1024];
	char *pkt_ptr = packet;
	char hash[16];
	int nwritten;
	struct packet_info pinfo;
	struct udp_register reg;
#ifdef IPSEC
	struct mse_ipsecext mnhasa;
#endif

	pinfo.reg = &reg;
#ifdef IPSEC
	pinfo.mnhasa = &mnhasa;
#endif


	
	/*  accept socket connection */ 
	sz = sizeof(struct sockaddr);
	newsd = accept( s, (struct sockaddr*)&harpRegRem, &sz );
	if ( newsd < 0 ){
		mip_log( MLOG_ERR, "HARP: harpRegDump(): accept failed\n" );
		if ( debug ){
			printf( "HARP: harpRegDump(): accept failed\n" );
		}
		return;
	}

	/*	recv dump request */
	harpReadSock( newsd, pkt_ptr, sizeof(short));

	type = ntohs(*((u_short*)pkt_ptr));
	pkt_ptr += sizeof(short);
	if ( type != HARP_DUMP_REQ ){
		mip_log( MLOG_ERR, "HARP: harpRegDump(): type failure\n" );
		return;
	}
	/*	read size */
	harpReadSock( newsd, pkt_ptr, sizeof(short));
	size = ntohs(*((u_short*)pkt_ptr));
	pkt_ptr += sizeof(short);
	if ( size != HARP_DUMP_REQ_SIZE ){
		mip_log( MLOG_ERR, "HARP: harpRegDUMP(): size failure\n" );
		return;
	}
#ifdef MIPSEC
	/*	read authentication */
	harpReadSock( newsd, pkt_ptr, 16 );
	if ( debug ){
		printf( "HARP: harpRegDump(): read auth (%x)\n", pkt_ptr );
	}
	computeMd5( packet, (pkt_ptr-packet), haha_key, hash );
	if ( bcmp( hash, pkt_ptr, 16 ) != 0){
		mip_log(MLOG_ERR, "HARP: harpRegDump(): auth failed h=(%x) p=(%x)\n", hash, pkt_ptr );
		return;
	}
#endif

	if ( debug ){
		printf("HARP: harpRegDump(): recvd reg dump request from co-HA\n" );
	}

	pkt_ptr = packet;	/* reset pointer */

	/* 	send dump message( T ) */
	type = HARP_REG_DUMP;
	type = htons( type );
	memcpy( pkt_ptr, (char*)&type, sizeof(short));
	nwritten = write( newsd, pkt_ptr, sizeof( short ));
	pkt_ptr += sizeof(short);
	if ( nwritten < sizeof( short )){
		mip_log( MLOG_ERR, "HARP: harpRegDump(): socket T write failure\n" );
		return;
	}
	/*	send size (L) */
	size = HARP_REG_DUMP_SIZE;
	size = htons( size );
	memcpy( pkt_ptr, (char*)&size, sizeof(short));
	nwritten = write( newsd, pkt_ptr, sizeof( short ));
	pkt_ptr += sizeof(short);
	if ( nwritten < sizeof( short )){
		mip_log( MLOG_ERR, "HARP: harpRegDump(): socket L write failure\n" );
	}
	if ( debug ){
		printf( "HARP:harpRegDump(): sent dump message\n" );
	}

	/* count entries in vlist */
	vlist_initwalk(hahead);
	while ( vlist_nextwalk()){
		count++;
	}
	/*  send number of vlist entries ( L ) */
	count = htonl( count );
	memcpy( pkt_ptr, (char*)&count, sizeof(long));
	nwritten = write( newsd, pkt_ptr, sizeof( long ));
	pkt_ptr += sizeof(long);
	if ( nwritten < sizeof( long )){
		mip_log( MLOG_ERR, "HARP: harpRegDump(): socket L write failure\n" );
		return;
	}
	if ( debug ){
		printf( "HARP:harpRegDump(): send vlist count: (%ld)\n", ntohl(count)); 
	}
	reg_size = sizeof( struct udp_register );
#ifdef IPSEC
	reg_size += sizeof( struct mse_ipsecext );
#endif
	reg_size = htonl( reg_size );
	memcpy( pkt_ptr, (char*)&reg_size, sizeof(long));
	nwritten = write( newsd, pkt_ptr, sizeof( long ));
	pkt_ptr += sizeof(long);
	if ( nwritten < sizeof( long )){
		mip_log( MLOG_ERR, "HARP: harpRegDump(): reg size message failed\n" );
		return;
	}
#ifdef MIPSEC
	/*	send hash */
	storeMd5( packet, (pkt_ptr-packet), hash, haha_key );
	nwritten = write( newsd, hash, 16 );
	if ( nwritten < 16 ){
		mip_log( MLOG_ERR, "HARP: harpRegDump(): write failed on hash\n");  
		return;
	}
	if ( debug ){
		printf("HARP: harpRegDump(): sent hash (%x)\n", hash );
	}
#endif
	
	/*  dump vlist ( V ) */
	vlist_initwalk(hahead);
	while( vptr = vlist_nextwalk()){
		struct udp_register *r;
#ifdef IPSEC
		struct mse_ipsecext *m;
#endif
		pkt_ptr = packet;
		r = (struct udp_register*)pkt_ptr;
		if ( debug ){
			printf( "HARP: harpRegDump(): dumping entry\n" );
		}
		pinfo.psize = sizeof( struct udp_register );
		vlist_to_reg( *vptr, r );
#ifdef IPSEC
		/* do ipsec */
		pinfo.psize += sizeof( struct mse_ipsecext );
		pkt_ptr += sizeof( struct udp_register );
		m = (struct mse_ipsecext*)pkt_ptr;
		vlist_to_ipsec( *vptr, m );
		if ( debug ){
			printf( "HARP: HarpRegDump(): type=%d length=%d 
					flag1=%x flag2=%x\n spi_ah=%ld 
					spi_esp=%ld\n",m->type,
				    m->length, m->flag1,
			     m->flag2, m->spi_ah, m->spi_esp );
		}
#endif
		pinfo.packet = packet;
		harpForward( newsd, &pinfo, HARP_REG_DUMP ); 
	}
}
	
void vlist_to_reg( struct vlist v, struct udp_register *reg )
{
	reg->type = UDP_REGISTER_TYPE;
	reg->flags = 0;		/* none supported */
	reg->lifetime = v.orig_lifetime;
	reg->home_address = v.mn;
	reg->home_agent = v.ha;
	reg->coa = v.coa;
	reg->i.l[0] = v.i.l[0];
	reg->i.l[1] = v.i.l[1];
}

#ifdef IPSEC
void vlist_to_ipsec( struct vlist v, struct mse_ipsecext *ipsc )
{
	struct keylist *key = NULL;

	key = search_keylist( mip_config.ipsec[HA_SECTYPE_MN].kl, v.mn.s_addr );
	if (( key == NULL ) || ( key->inuse == FALSE )){
		ipsc->flag2 = 0;	
	}
	else {
		ipsc->flag2 = key->transform;
		ipsc->spi_ah = htonl( key->spi_ah[ SPI_INB ]);
		ipsc->spi_esp = htonl( key->spi_esp[ SPI_INB ]); 
	}
	ipsc->type = MN_HA_IPSEC_POLICY_TYPE;
	ipsc->length = 2 + 8; /* transform type + two spis */
	ipsc->flag1 = 0;
}
#endif


/*   send dump request byte then recieve registration table, *
 *	create pinfo structure & forward to haUdp()          
    0               1               2               3
    0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
   |     type=HARP_DUMP_REQ         |        size=0                |
   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
*/
void recvHarpDump( int s )
{
	char buf[1024];
	char *buf_ptr = buf;
	char hash[1024];
	int i, nwritten;
	int auth = TRUE;
	unsigned short type, size;
	unsigned long count=0, reg_size;
	struct packet_info pinfo;

	/*	write dump request */
	type = HARP_DUMP_REQ;
	type = htons( type );
	memcpy( buf_ptr, (char*)&type, sizeof(short));
	nwritten = write( s, buf_ptr, sizeof(short));
	buf_ptr += sizeof(short);
	if ( nwritten < sizeof(short) ){
		mip_log( MLOG_ERR, "HARP: recvHarpDump(): socket write error\n" );
		return;
	}
	/*  write message size */
	size = HARP_DUMP_REQ_SIZE;
	size = htons( size );
	memcpy( buf_ptr, (char*)&size, sizeof(short));
	nwritten = write( s, buf_ptr, sizeof( short ));
	buf_ptr += sizeof(short);
	if ( nwritten < sizeof( short )){
		mip_log( MLOG_ERR, "HARP: recvHarpDump(): socket write error\n" );
		return;
	}
#ifdef MIPSEC
	storeMd5( buf, (buf_ptr-buf), hash, haha_key );
	nwritten = write( s, hash, 16 );
	if ( nwritten < 16 ){
		mip_log( MLOG_ERR, "HARP: recvHarpDump(): socket write error\n" );
	}

	if ( debug ){
		printf( "HARP: recvHarpDump(): hash= (%x)\n", hash );
		printf( "HARP: recvHarpDump(): wrote dump request\n" );
	}
#endif

	buf_ptr = buf;	/* reset buf pointer */

	/*	read dump message (T) */
	harpReadSock( s, buf_ptr, sizeof(short));
	type = ntohs(*((u_short*)buf_ptr));
	buf_ptr += sizeof(short);
	if ( type != HARP_REG_DUMP ){
		mip_log( MLOG_ERR, "HARP: recvHarpDump(): dump message error\n" );
		return;
	}

	/*	read message size (L) */
	harpReadSock( s, buf_ptr, sizeof(short));
	size = ntohs(*((u_short*)buf_ptr));
	buf_ptr += sizeof(short);
	if ( size != HARP_REG_DUMP_SIZE ){
		mip_log( MLOG_ERR, "HARP: recvHarpDump(): dump messages error\n" );
		return;
	}

	/* 	read in registration count */
	harpReadSock( s, buf_ptr, sizeof( long ));
	count = ntohl(*((u_long*)buf_ptr));
	buf_ptr += sizeof(long);
	if ( debug ){
		printf( "HARP: recvHarpDump(): read in vlist count (%ld)\n", count );
	}
	/*	read in registration size */
	harpReadSock( s, buf_ptr, sizeof( long ));
	reg_size = ntohl(*((u_long*)buf_ptr));
	buf_ptr += sizeof(long);
	if ( reg_size != ( sizeof( struct udp_register )
#ifdef IPSEC
				+sizeof( struct mse_ipsecext)
#endif
			)){
		mip_log( MLOG_ERR, "recvHarpDump(): reg size wrong\n" );
		return;
	}
#ifdef MIPSEC
	/*	read authentication */
	harpReadSock( s, buf_ptr, 16 );
	computeMd5( buf, (buf_ptr-buf), haha_key, hash );
	if ( bcmp( hash, buf_ptr, 16 ) != 0){
		mip_log( MLOG_ERR, "recvHarpDump(): auth failed, ignoring registrations\n" );
		auth = FALSE;
	}
#endif
	

	/* read in registrations */
	for (i=0; i< count; i++ ){
		u_short t, sz;
		char *packet; 

		buf_ptr = buf;
		if ( debug ){
			printf("HARP: recvHarpDump(): reading registration \n");
		}

		/* read type */
		harpReadSock( s, buf_ptr, sizeof( short ));
		t = ntohs( *((u_short*)buf_ptr ));
		buf_ptr += sizeof(short);
		if ( debug ){
			printf("HARP: recvHarpDump(): read type %d\n", t );
		}
		/* read size */
		harpReadSock( s, buf_ptr, sizeof( short ));
		sz = ntohs( *((u_short*)buf_ptr ));
		buf_ptr += sizeof(short);
		if ( debug ){
			printf("HARP: recvHarpDump(): read size %d\n", sz );
		}
		/* read data */
		harpReadSock( s, buf_ptr, sz );
		if ( debug ){
			printf("HARP: dumping read registration data\n");
			dumpme( buf_ptr, sz );
		}
		packet = buf_ptr;
		buf_ptr += sz;
		if ( debug ){
			printf("HARP: recvHarpDump(): read data\n" );
		}
#ifdef MIPSEC
		/* read hash */
		harpReadSock( s, buf_ptr, 16);
		computeMd5( buf, (buf_ptr-buf), haha_key, hash );
		if ( bcmp( hash, buf_ptr, 16 ) != 0){
			mip_log(MLOG_ERR, "HARP: recvHarpDump() auth failed, ignoring all further registrations\n" );
			auth = FALSE;
		}
		if ( debug ){
			printf( "recvHarpDump: compared hashes (%x) (%x)\n" ,hash, buf_ptr );
		}
#endif

		if ( t != HARP_FORWARD ){
			mip_log( MLOG_ERR, "recvHarpDump(): wrong type\n" );
			return; 
		}
		
		pinfo.packet = packet;
		pinfo.psize = sz;

		pinfo.reg = (struct udp_register*)packet;
#ifdef IPSEC
		pinfo.mnhasa = (struct mse_ipsecext*)(packet + sizeof( struct udp_register ));
		pinfo.mnhasa_size = sizeof( struct mse_ipsecext );
		pinfo.mnhasa_denied = FALSE;
		pinfo.faxsa = NULL;
		pinfo.faxsa_size = 0;
		/* pinfo.mnhasa->spi_ah = ntohl( pinfo.mnhasa->spi_ah );
		pinfo.mnhasa->spi_esp = ntohl( pinfo.mnhasa->spi_esp ); */
		if ( debug ){
			printf( "HARP: recvHarpDump(): type=%d length=%d 
					flag1=%x flag2=%x\n spi_ah=%ld 
					spi_esp=%ld\n", pinfo.mnhasa->type,
				    pinfo.mnhasa->length, pinfo.mnhasa->flag1,
			     pinfo.mnhasa->flag2, pinfo.mnhasa->spi_ah, pinfo.mnhasa->spi_esp );
		}
					 
#endif
		pinfo.fa_x_auth = 0;
		pinfo.fa_x_auth_size = 0;
		pinfo.fa_x_auth_count = 0;
		pinfo.postmnha_size = 0;
		pinfo.forward = TRUE;

		/* call haUdp() with this packet */
		if ( debug ){
			printf( "HARP: recvHarpDump(): calling haUdp()\n" );
		}
		if ( auth == TRUE ){
			haUdp( NULL, &harpRegRem, &pinfo, NULL );
		}
		else if ( debug ){
			printf(" recvHarpDump(): ignoring reg due to bad auth\n" );
		}
	}
}

/*	read size bytes from tcp socket */
void harpReadSock( int sd, char *buf, int size )
{
	int nread;

	while( size ){
		nread = read( sd, buf, size );
		if ( nread < 0 ){
			if ( errno == EWOULDBLOCK && debug ){
				printf( "HARP: harpReadSock(): would have blocked on read\n" );
			}
			mip_log( MLOG_ERR, "HARP: harpReadSock(): socket read error\n ");
			return;
		}
		buf += nread;
		size -=nread;
	}
	return;
}

/* forward UDP registration packet to co-HA
    0               1               2               3
    0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
   |     type=HARP_FORWARD          |     size                     |
   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
   |                Mobile-IP registration packet ...              |
   |                                                               |

*/
void harpForward( int sock, struct packet_info *pinfo, int t )
{
	char buf[ 1024 ];
	char *buf_ptr = buf;
	u_short type, length;
	u_long tmp;
	/* debugging */
#ifdef IPSEC
	struct mse_ipsecext *mse;
#endif



	if ( debug ){
		printf( "HARP: harpForward(): forwarding reg. packet to %s: %d\n",
			inet_ntoa( mip_config.coha.sin_addr ), HARP_PORT );
	}

	
	type = htons( HARP_FORWARD );/*	  (T) */  
	memcpy( buf_ptr, (char*)&type, sizeof(short));
	buf_ptr += sizeof(short);

	
	length = pinfo->psize;  /*  (L)  */
	if ( debug ){
		printf( "HARP: harpForward(): added type, size=%d\n", length );
	}
	tmp = htons( length );
	memcpy( buf_ptr, (char*)&tmp, sizeof(short)); 
	
	buf_ptr += sizeof(short);

	memcpy( buf_ptr, pinfo->packet, pinfo->psize );
	if ( debug ){
		printf( "HARP: dump of message to be forwarded\n" );
		dumpme( buf_ptr, pinfo->psize );
	}
#ifdef IPSEC
	if ( debug ){
		mse = (struct mse_ipsecext*)(buf_ptr + sizeof( struct udp_register ));
		printf( "HARP: harpForward(): type=%d length=%d
                                        flag1=%x flag2=%x\n spi_ah=%ld
                                        spi_esp=%ld\n", mse->type,
                                    mse->length, mse->flag1,
                             mse->flag2, mse->spi_ah, mse->spi_esp );
	}
#endif
	
	buf_ptr += pinfo->psize;

#ifdef MIPSEC
	storeMd5( buf, (buf_ptr-buf), buf_ptr, haha_key );
	if ( debug ){
		printf( "dumping hash:\n" );
		dumpme( buf_ptr, 16 );
	}
	buf_ptr += 16;
#endif

	if ( t == HARP_FORWARD ){
		sendOneUdp( harp_sendSock, buf, (buf_ptr - buf), 
			mip_config.coha.sin_addr, htons(HARP_PORT ));
	}
	else if ( t == HARP_REG_DUMP ){
		write( sock, buf, (buf_ptr - buf));
	} 
		
}

/*
    0               1               2               3
    0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
   |     type=HARP_PING             |       size=4                 |
   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
   |     HARP peer IP address                          |
   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
*/

void harpPing()
{
	char buf[1024];
	char *buf_ptr = buf;
	u_short type, length, tmp;
	u_long my_ip;

	type = htons( HARP_PING );	/*  (T)  */
	memcpy( buf_ptr, (char*)&type, sizeof( short ));
	buf_ptr += sizeof( short );

	length = HARP_PING_SIZE;  /* (L) */
	tmp = htons( length );
	memcpy( buf_ptr, (char*)&tmp, sizeof( short ));
	buf_ptr += sizeof( short );

	/* HA ip */
	my_ip = getHaIp();
	if ( my_ip == 0 ){
		mip_log( MLOG_ERR, "HARP: harpPing, failed to get HA IP\n" );
		return;
	}
	my_ip = htonl( my_ip );
	memcpy( buf_ptr, (char*)&my_ip, sizeof(long)); 
	buf_ptr += sizeof( long );

#ifdef MIPSEC
	/* auth hash */
	storeMd5( buf, (buf_ptr - buf), buf_ptr, haha_key );
	buf_ptr += 16;
#endif

	if ( debug ) {
		printf( "HARP: harpPing(): sending harp ping\n" );
	}
	
	sendOneUdp( harp_sendSock, buf, (buf_ptr - buf), mip_config.coha.sin_addr, htons(HARP_PORT));
}

/*
    0               1               2               3
    0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
   |     type=HARP_ACK              |        size=4                |
   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
   |     HARP peer IP address                          |
   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
*/

void harpAck()
{
	char buf[1024];
	char *buf_ptr = buf;
	u_short type, length, tmp;
	u_long my_ip;

	type = htons( HARP_ACK );
	memcpy( buf_ptr, (char*)&type, sizeof( short ));
	buf_ptr += sizeof( short );

	length = 0;  /*  (L)  */
	tmp = htons( length );
	memcpy( buf_ptr , (char*)&tmp, sizeof( short ));
	buf_ptr += sizeof( short );

	/* 	HA ip */
	my_ip = getHaIp();
	if ( my_ip == 0 ){
		mip_log( MLOG_ERR, "HARP: harpAck, failed to get HA IP\n" );
		return;
        }
        my_ip = htonl( my_ip );
        memcpy( buf_ptr, (char*)&my_ip, sizeof(long));
	buf_ptr += sizeof( long );

#ifdef MIPSEC
	/* auth hash */
	storeMd5( buf, (buf_ptr - buf), buf_ptr, haha_key );
	buf_ptr += 16;
#endif

	if ( debug ) {
		printf( "HARP: harpAck(): sending harp ACK\n" );
	}

	sendOneUdp( harp_sendSock, buf, (buf_ptr - buf) , mip_config.coha.sin_addr, htons( HARP_PORT ));
}


/* turn off arp on interface for secodary HA */
#define ON 1
#define OFF 0
void harp2ArpOn(){ harp2Arp(ON); }
void harp2ArpOff(){ harp2Arp(OFF); }

void harp2Arp( int arg )
{

	/* if option is not on, ignore these calls */
	if ( !mip_config.harp2_arp_disable ){
		return;
	}
	if ( arg == OFF ){
		if ( debug ){
			printf( "HARP: harp2Arp(): turning arp off on interface %s\n",  mip_config.harp2_if );
		}
		mip_log( MLOG_INFO, "Turning arp off on interface %s\n", mip_config.harp2_if );
		set_arp( mip_config.harp2_if, "-arp", 128 );
	} else {
		if ( debug ){
			printf( "HARP: harp2Arp(): turning arp on on interface %s\n", mip_config.harp2_if );
		}
		mip_log( MLOG_INFO, "Turning arp on on interface %s\n", mip_config.harp2_if );
		set_arp( mip_config.harp2_if, "arp", -128 );
	}
}
	
#endif /* HARP */
