
/*
 * Copyright (c) 1996 Portland State University 
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Portland State University.
 * 4. The name of Portland State University may not be used to endorse 
 *    or promote products derived from this software without specific 
 *    prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR/S ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR/S BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* upd registration request and reply
*/

#define MAXUDPPACKET	1500

struct udp_register {
#define UDP_REGISTER_TYPE	1
	u_char type; 	/* type 1 is registration request */

#define S_BIT  0x80     /* mn requests ha to retain sim. bindings */
#define B_BIT  0x40	/* mn requests forwarding of broadcasts */
#define D_BIT  0x20	/* mn using coa, will do own decaps */
#define M_BIT  0x10     /* mn requests minimal encapsulation */
#define G_BIT  0x08	/* mn requests GRE encapsulation */
#define V_BIT  0x04	/* mn requests Van Jacobson compression */
#define R1_BIT 0x02	/* reserved. set as zero */
#define R2_BIT 0x01	/* reserved, set as zero */

	u_char flags;
	u_short lifetime;	/* mn requests lifetime. 0 means dereg */
	struct in_addr home_address; /* mn's home address */
	struct in_addr home_agent;   /* ha's address */
	struct in_addr coa;	     /* coa being used */
	union ident {
/* TBD. look at linux versions */
		u_long l[2];
		u_char c[8];
	} i;
};

struct udp_reply {
#define UDP_REPLY_TYPE	3
	u_char type; 	/* type 3 is registration reply */
	u_char code;    /* success/failure code */	
	u_short lifetime;	/* mn requests lifetime. 0 means dereg */
	struct in_addr home_address; /* mn's home address */
	struct in_addr home_agent;   /* ha's address */
	union ident_r {
		u_long l[2];
		u_char c[8];
	} i;
};

/****************************************************/
/* reply codes - registration accepted */
#define REG_REPLY_OK			0 
/* ok but can't do mult. mobile bindings */
#define REG_OK_NOMB			1	

/****************************************************/
/* fa denies registration */
/* ok but can't do mult. mobile bindings */
#define REG_FAIL_FA_UNSPEC		64
#define REG_FAIL_FA_PROHIB		65
#define REG_FAIL_FA_RESOURCES		66
#define REG_FAIL_FA_MN_AUTH		67
#define REG_FAIL_FA_HA_AUTH		68
#define REG_FAIL_FA_LIFETIME		69
#define REG_FAIL_FA_REQUEST		70
#define REG_FAIL_FA_REPLY		71
#define REG_FAIL_FA_ENCAPS		72 
#define REG_FAIL_FA_VJ			73 
#define REG_FAIL_FA_NETUNREACH		80 
#define REG_FAIL_FA_HAUNREACH		81 
#define REG_FAIL_FA_PORTUNREACH		82 
#define REG_FAIL_FA_OTHERUNREACH	88 
#define REG_IS_FAIL_FA(code) \
	((code) >= REG_FAIL_FA_UNSPEC &&\
		(code) <= REG_FAIL_FA_OTHERUNREACH)

/****************************************************/
/* ha denies registration */
#define REG_FAIL_HA_UNSPEC		128
#define REG_FAIL_HA_PROHIBIT		129
#define REG_FAIL_HA_RESOURCES		130
#define REG_FAIL_HA_MN_AUTH		131
#define REG_FAIL_HA_FA_AUTH		132
#define REG_FAIL_HA_IDMISMATCH		133
#define REG_FAIL_HA_REQUEST		134
#define REG_FAIL_HA_BINDINGS		135
#define REG_FAIL_HA_UNKNOWN		136
    
/* mobile-home authentication extension
*/
#define MSEHASHOFFSET	6

#ifdef NOSPIHASH
#define AUTHEXT_HASHEDBYTES		2
#else
#define AUTHEXT_HASHEDBYTES		MSEHASHOFFSET
#endif

struct mse_authext {
#define MN_HA_AUTHENTICATOR_TYPE	32
#define MN_FA_AUTHENTICATOR_TYPE	33
#define FA_HA_AUTHENTICATOR_TYPE	34
	u_char type;
	u_char length;			/* length of data + spi */
/* spi is a long, but it won't end up aligned right as
 * there is only a short before it.
 * at least this way the address is right for mse->spi
 * as an address.
 */
	u_char spi;			/* security parameter index */
	u_char spipad[3];
	u_char hash[16];		/* enough room for the rest */
};

#ifdef IPSEC
/*
 *  MIP MH-HA SPI exchange:
 *
 *  A message is sent from the MN to the HA when a relationship
 *  using IPSec transforms is desired by the MN.  It is assumed that the
 *  spi and key information is preconfigured in both hosts, this exchange
 *  simply chooses a specific spi and a specific transform type.  The HA
 *  either changes the the LL/tunnel to the MN in accordance with the
 *  request from the MN or reports that the request cannot be met.
 *  Currently the only HA-based policy decision is to require an IPSec
 *  relationship, thus locking out MNs which do not request one.
 *
 *  Important fields in struct mse_ipsecext are: spi_ah and spi_esp containing
 *  preconfigured AH and/or ESP spi's; and, 16 flag bits which indicate
 *  which spi_* field is set, if any.  Currently only the lower two bits
 *  are used defined in "ipsec.h" as AHSEC and ESPSEC.  A MN may request
 *  AH or ESP by setting the appropriate bit and spi field, it may
 *  request AH+ESP by setting both bits and both fields (the spi's may be
 *  different and are still logically separate), or it may request no
 *  security by setting all bits and fields to zero.  The latter is
 *  equivalent to removing a secure channel if one already exists.
 *
 *  Agents cache security characteristics for a given MN in their visitor
 *  list entries.  MNs cache security characteristics for agents in the
 *  agent list entries when both away (the FA vlist) and at home (the HA
 *  vlist).  MNs cache security characteristics for other MNs in their
 *  "mn lists."
 *
 *  Currently secure channels may be formed between agents and MN's and
 *  between individual MN's.  When one host first recognizes another,
 *  the secure route SA (rosa) is immediately cached in the appropriate
 *  list entry if one exists in the Mobile-IP configuration file for that
 *  host and turned "on."  All SAs indicated
 *  in the mipd.conf and mnd.conf files are assumed to be preloaded into
 *  the kernel using keyadmin(8).  No key bits are kept in the Mobile-IP
 *  configuration files.  If spi's exist for both AH and ESP, both
 *  are configured (TBD).  rosa information is then unchanged unless the
 *  MN chooses otherwise.  A MN may choose to lessen or remove a secure
 *  channel with the HA (configured by default in the HA when the MN first
 *  registers) by sending the appropriate mse_ipsecext message.
 *  Between MNs, if one chooses to modify a secure channel the change is
 *  only unidirectional, ie, the MN which did not initiate the change does
 *  not remove, change, or upgrade it's secure channel when it's peer
 *  chooses to make a change.  Eventually there may be policy between
 *  two (or a group of) MN's to coordinate this (TBD)).
 */

/*
 * mn can send ipsec policy extension to HA or FA.
 * HA will ack by echo of policy tlv or may NAK
 * with REG_FAIL_HA_PROHIBIT.  If HA does not
 * return TLV, mn should assume NO ipsec support.
 *
 * flags are named in ipsec.h
 */
#define MN_HA_IPSEC_POLICY_TYPE		192
#define MN_FA_IPSEC_POLICY_TYPE		193
struct mse_ipsecext {
	u_char type;
	u_char length;
	u_char flag1;	/* upper byte: 15-8 */
	u_char flag2;	/* lower byte: 7-0 */
	u_long	spi_ah,
		spi_esp;
};
#endif
