/*
 * Copyright (c) 1996 Portland State University 
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Portland State University.
 * 4. The name of Portland State University may not be used to endorse 
 *    or promote products derived from this software without specific 
 *    prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR/S ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR/S BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * getmac.c
 *
 * function to take ifname as input and return mac address
 * Linux version is much shorter thanks to SIOGIFHWADDR.
 */
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <linux/if.h>
#include <string.h>
#ifdef TESTMAIN
#include <stdio.h>
#endif

#define MACMSIZ 30


/*
 * given an interface name, return a mac address
 *
 * NULL: on error
 */
char *
getmac(char ifname[] )
{
	u_char *mac_addr;
	struct ifreq ifr;
	int sd;
	int rc;

	mac_addr=(char *)malloc(MACMSIZ*sizeof(char));
	if (mac_addr == NULL)
		return( NULL );
	sd = socket( AF_INET, SOCK_DGRAM, 0 );
	if ( sd < 0 ){
		return NULL;
	}
	strcpy( ifr.ifr_name, ifname );
	rc = ioctl( sd, SIOCGIFHWADDR, &ifr );
	if ( rc < 0 ){
		close(sd);
		return NULL;
	}
	memcpy( mac_addr, ifr.ifr_hwaddr.sa_data, 8 );
	return mac_addr;
}


#ifdef TESTMAIN
main(argc, argv)
	int argc;
	char *argv[];
{
	int i;
	u_char *mac_addr;
	if (argc<2) {
		fprintf(stderr, "usage: ether if_name\n");
		exit(1);
	}
	if (mac_addr=getmac(argv[1])) {
		printf("MAC addr for %s is ", argv[1]);
		for (i=0;i<6;i++) 
			printf("%02x%c", *mac_addr++ & 0xff, (i<5)?':':' ');
		printf("\n");
	}
	else printf("No mac addr for %s\n", argv[1]);
}
#endif

