
/*
 * Copyright (c) 1996 Portland State University 
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Portland State University.
 * 4. The name of Portland State University may not be used to endorse 
 *    or promote products derived from this software without specific 
 *    prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR/S ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR/S BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdio.h>
#include <sys/types.h>
#include <sys/uio.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <sys/types.h>
#include <sys/times.h>
#include <net/if.h>
/*#include <net/if_dl.h> */
#include <linux/un.h>
#include <errno.h>
#include <signal.h>

extern int debug;

/*
 * interface to recvmsg system call.  We assume
 * that it returns a RECVIF control structure internally.
 *
 * note: index == 0 is not possible.
 *
 * returns:
 *	buf - packet itself with size returned directly (rc) 
 *	from - sockaddr_in for incoming packet
 *	ifindex - value of interface
 */
int recvMsg(int sd, char *buf, int bufsize, struct sockaddr_in *from, 
	int *fromsize, int *ifindex)     

{
	int rc;
	char controlbuf[1024];
	struct msghdr msg;
	struct iovec i;
	extern int errno;
	struct cmsghdr *cmsg;

	msg.msg_name = (caddr_t) from;
	msg.msg_namelen = *fromsize;
	i.iov_base = buf;
	i.iov_len = bufsize;
	msg.msg_iov = &i;
	msg.msg_iovlen = 1;
	msg.msg_control = controlbuf;
	msg.msg_controllen = 1024;

	errno = 0;
#ifdef SAVEENERGY
	rc = RECVMSG(sd, &msg, 0);
#else
	rc = recvmsg(sd, &msg, 0);
#endif
	if (rc < 0) {
		return(-1);
	}
	*ifindex = 1;
	return(rc);
}
